searchData={"items":[{"type":"module","title":"EXLA","doc":"[Google's XLA](https://www.tensorflow.org/xla/) (Accelerated Linear Algebra) compiler/backend for Nx.\n\nIt supports just-in-time (JIT) compilation to GPU (both CUDA and ROCm) and TPUs.","ref":"EXLA.html"},{"type":"module","title":"XLA binaries - EXLA","doc":"EXLA relies on the [XLA](https://github.com/elixir-nx/xla) package to\nprovide the necessary XLA binaries. Whenever possible it tries to download\nprecompiled builds, but you may need to build from source if there is no\nversion matching your target environment. For more details, including\nGPU/TPU support see [the usage section](https://github.com/elixir-nx/xla#usage).","ref":"EXLA.html#module-xla-binaries"},{"type":"module","title":"Configuration - EXLA","doc":"EXLA ships with a backend to store tensors and run computations on.\nGenerally speaking, the backend is enabled globally in your `config/config.exs`\n(or `config/ENV.exs`) with the following:\n\n    import Config\n    config :nx, :default_backend, EXLA.Backend\n\nIn a script/notebook, you would do:\n\n    Mix.install([\n      {:exla, \"~> 0.2\"}\n    ])\n\n    Nx.global_default_backend(EXLA.Backend)\n\nFrom now on, all created tensors will be allocated directly on the given\n`EXLA.Backend`. You can use functions such as `Nx.backend_transfer/2` to\nexplicitly transfer tensors.\n\nEXLA will pick an available client to allocate and compute tensors, in this\norder: `:cuda`, `:rocm`, `:tpu`, and `:host` (CPU). See the \"Clients\" section\nbelow for more information.\n\nTo use GPUs/TPUs, you must also set the appropriate value for the\n[`XLA_TARGET`](https://github.com/elixir-nx/xla#xla_target) environment\nvariable. If you have GPU/TPU enabled, we recommend setting the environment\nvariable for your machine altogether. For CUDA, setting\n`ELIXIR_ERL_OPTIONS=\"+sssdio 128\"` is also required on more complex operations\nto increase CUDA's compiler stack size.\n\nNote that setting the `EXLA.Backend` does not enable the EXLA compiler.\nYou must still pass the `compiler: EXLA` option to `Nx.Defn` functions\nor call the functions in this module.","ref":"EXLA.html#module-configuration"},{"type":"module","title":"Options - EXLA","doc":"The options accepted by EXLA backend/compiler are:\n\n  * `:client` - an atom representing the client to use. The default\n    client is chosen on this order: `:cuda`, `:rocm`, `:tpu`, and `:host`.\n\n  * `:device_id` - the default device id to run the computation\n      on. Defaults to the `:default_device_id` on the client\n\n  * `:precision` - control the tradeoff between speed and accuracy for\n    array computations on accelerator backends (i.e. TPU and GPU).\n    It must be one of:\n\n    * `:default` - Fastest mode, but least accurate. Performs computations\n      in bfloat16\n\n    * `:high` - Slower but more accurate. Performs float32 computations in\n      3 bfloat16 passes, or using tensorfloat32 where available\n\n    * `:highest` - Slowest but most accurate. Performs computations in float32\n      or float64 as applicable\n\n  * `:compiler_mode` - the mode to use for the compiler. It must be one of:\n\n    * `:mlir` - The default mode. Uses MLIR to compile the computation\n\n    * `:xla` - Legacy implementation. Uses XLA to compile the computation\n\n  The `:compiler_mode` can also be set globally through a specific fallback config:\n\n      config :exla, :compiler_mode, :mlir","ref":"EXLA.html#module-options"},{"type":"module","title":"Clients - EXLA","doc":"The `EXLA` library uses a client for compiling and executing code.\nThose clients are typically bound to a platform, such as CPU or\nGPU.\n\nThose clients are singleton resources on Google's XLA library,\ntherefore they are treated as a singleton resource on this library\ntoo. EXLA ships with runtime client configuration for each supported\nplatform:\n\n    config :exla, :clients,\n      cuda: [platform: :cuda],\n      rocm: [platform: :rocm],\n      tpu: [platform: :tpu],\n      host: [platform: :host]\n\nIn a script/notebook, you can set those after `Mix.install/2`,\nbut before any tensor operation is performed:\n\n    Application.put_env(:exla, :clients,\n      cuda: [platform: :cuda],\n      rocm: [platform: :rocm],\n      tpu: [platform: :tpu],\n      host: [platform: :host]\n    )\n\nYou can provide your own list of clients, replacing the list above\nor configuring each client as listed below. You can also specify\n`:default_client` to set a particular client by default or\n`:preferred_clients` to change the order of clients preference,\nbut those configurations are rarely set in practice.\n\n> **Important!** you should avoid using multiple clients for the\n> same platform. If you have multiple clients per platform, they\n> can race each other and fight for resources, such as memory.\n> Therefore, we recommend developers to stick with the default\n> clients above.","ref":"EXLA.html#module-clients"},{"type":"module","title":"Client options - EXLA","doc":"Each client configuration accepts the following options:\n\n  * `:platform` - the platform the client runs on. It can be\n    `:host` (CPU), `:cuda`, `:rocm`, or `:tpu`. Defaults to `:host`.\n\n  * `:default_device_id` - the default device ID to run on.\n    For example, if you have two GPUs, you can choose a different\n    one as the default. Defaults to device 0 (the first device).\n\n  * `:preallocate`- if the memory should be preallocated on\n    GPU devices. Defaults to `true`.\n\n  * `:memory_fraction` - how much memory of a GPU device to\n    allocate. Defaults to `0.9`.","ref":"EXLA.html#module-client-options"},{"type":"module","title":"Memory preallocation - EXLA","doc":"XLA preallocates memory in GPU devices. This means that, if you are to\nrun multiple notebooks or multiple instances of your application, the\nsecond, third, and so on instances won't be able to allocate memory.\n\nYou can disable this behaviour by setting `preallocate: false` on the\nclient configuration, as specified above. You may also use\n`:memory_fraction` to control how much is preallocated.","ref":"EXLA.html#module-memory-preallocation"},{"type":"module","title":"GPU Runtime Issues - EXLA","doc":"GPU Executions run in dirty IO threads, which have a considerable smaller\nstack size than regular scheduler threads. This may lead to problems with\ncertain CUDA or cuDNN versions, leading to segmentation fails. In a development\nenvironment, it is suggested to set:\n\n    ELIXIR_ERL_OPTIONS=\"+sssdio 128\"\n\nTo increase the stack size of dirty IO threads from 40 kilowords to\n128 kilowords. In a release, you can set this flag in your `vm.args`.","ref":"EXLA.html#module-gpu-runtime-issues"},{"type":"module","title":"Docker considerations - EXLA","doc":"EXLA should run fine on Docker with one important consideration:\nyou must not start the Erlang VM as the root process in Docker.\nThat's because when the Erlang VM runs as root, it has to manage\nall child programs.\n\nAt the same time, Google XLA's shells out to child programs and\nmust retain control over how child programs terminate.\n\nTo address this, simply make sure you wrap the Erlang VM in\nanother process, such as the shell one. In other words, if you\nare using releases, instead of this:\n\n    CMD path/to/release start\n\ndo this:\n\n    CMD sh -c \"path/to/release start\"\n\nIf you are using Mix inside your Docker containers, instead of this:\n\n    CMD mix run\n\ndo this:\n\n    CMD sh -c \"mix run\"\n\nAlternatively, you can pass the `--init` flag to `docker run`,\nso it runs an `init` inside the container that forwards signals\nand reaps processes.\n\nThe `--init` flag uses the [`tini`](https://github.com/krallin/tini)\nproject, so for cases where the flag may not available (e.g.\nkubernetes) you may want to install it.","ref":"EXLA.html#module-docker-considerations"},{"type":"module","title":"Telemetry events - EXLA","doc":"EXLA executes a telemetry event every time a function is JIT-compiled.\nThe events are named `[:exla, :compilation]` and include the following\nmeasurements, given in microseconds:\n\n  * `:eval_time` - the time spent on turning the function into XLA\n    computation\n  * `:compile_time` - the time spent on compiling the XLA computation\n    into an executable\n  * `:total_time` - the sum of `:eval_time` and `:compile_time`\n\nThe metadata is:\n\n  * `:key` - the compilation key for debugging","ref":"EXLA.html#module-telemetry-events"},{"type":"function","title":"EXLA.cached?/3","doc":"Checks if the compilation of function with args is cached.\n\nNote that hooks are part of the cache, and\ntherefore they must be included in the options.","ref":"EXLA.html#cached?/3"},{"type":"function","title":"Examples - EXLA.cached?/3","doc":"iex> fun = fn a, b -> Nx.add(a, b) end\n    iex> left = Nx.tensor(1, type: {:u, 8})\n    iex> right = Nx.tensor([1, 2, 3], type: {:u, 16})\n    iex> EXLA.jit(fun).(left, right)\n    iex> EXLA.cached?(fun, [left, right])\n    true\n    iex> EXLA.cached?(fun, [left, Nx.tensor([1, 2, 3, 4], type: {:u, 16})])\n    false\n\nCompiled functions are also cached, unless cache is set to false:\n\n    iex> fun = fn a, b -> Nx.subtract(a, b) end\n    iex> left = Nx.tensor(1, type: {:u, 8})\n    iex> right = Nx.tensor([1, 2, 3], type: {:u, 16})\n    iex> EXLA.compile(fun, [left, right], cache: false)\n    iex> EXLA.cached?(fun, [left, right])\n    false\n    iex> EXLA.compile(fun, [left, right])\n    iex> EXLA.cached?(fun, [left, right])\n    true","ref":"EXLA.html#cached?/3-examples"},{"type":"function","title":"EXLA.compile/3","doc":"A shortcut for `Nx.Defn.compile/3` with the EXLA compiler.\n\n    iex> fun = EXLA.compile(&Nx.add(&1, &1), [Nx.template({3}, {:s, 64})])\n    iex> fun.(Nx.tensor([1, 2, 3]))\n    #Nx.Tensor \n\nResults are allocated on the `EXLA.Backend`. Note that the\n`EXLA.Backend` is asynchronous: operations on its tensors\n*may* return immediately, before the tensor data is available.\nThe backend will then block only when trying to read the data\nor when passing it to another operation.","ref":"EXLA.html#compile/3"},{"type":"function","title":"Options - EXLA.compile/3","doc":"It accepts the same option as `Nx.Defn.compile/3` plus:\n\n  * `:debug` - print compile and debugging information, defaults to `false`.\n\n  * `:cache` - cache the results of compilation, defaults to `true`.\n    You can set it to false if you plan to compile the function only\n    once and store the compile contents somewhere.\n\n  * `:client` - an atom representing the client to use. The default\n    client is chosen on this order: `:cuda`, `:rocm`, `:tpu`, and `:host`.\n\n  * `:device_id` - the default device id to run the computation on.\n    Defaults to the `:default_device_id` on the client","ref":"EXLA.html#compile/3-options"},{"type":"function","title":"EXLA.jit/2","doc":"A shortcut for `Nx.Defn.jit/2` with the EXLA compiler.\n\n    iex> EXLA.jit(&Nx.add(&1, &1)).(Nx.tensor([1, 2, 3]))\n    #Nx.Tensor \n\nResults are allocated on the `EXLA.Backend`. Note that the\n`EXLA.Backend` is asynchronous: operations on its tensors\n*may* return immediately, before the tensor data is available.\nThe backend will then block only when trying to read the data\nor when passing it to another operation.","ref":"EXLA.html#jit/2"},{"type":"function","title":"Options - EXLA.jit/2","doc":"It accepts the same option as `Nx.Defn.jit/2` plus:\n\n  * `:cache` - cache the results of compilation, defaults to `true`.\n\n  * `:client` - an atom representing the client to use. The default\n    client is chosen on this order: `:cuda`, `:rocm`, `:tpu`, and `:host`.\n\n  * `:debug` - print compile and debugging information, defaults to `false`.\n\n  * `:device_id` - the default device id to run the computation on.\n    Defaults to the `:default_device_id` on the client\n\n  * `:lazy_transfers` - when `:always`, it lazily transfers data to the device\n    instead of upfront. This is useful to reduce memory allocation on GPU/TPU\n    devices at the cost of increased latency. **It is recommended to only enable\n    this if the input tensors are allocated on host and the computation is\n    running on GPU/TPU with a limited amount of memory**","ref":"EXLA.html#jit/2-options"},{"type":"function","title":"EXLA.jit_apply/3","doc":"A shortcut for `Nx.Defn.jit_apply/3` with the EXLA compiler.\n\n    iex> EXLA.jit_apply(&Nx.add(&1, &1), [Nx.tensor([1, 2, 3])])\n    #Nx.Tensor \n\nSee `jit/2` for supported options.","ref":"EXLA.html#jit_apply/3"},{"type":"function","title":"EXLA.stream/3","doc":"Starts streaming the given anonymous function with just-in-time\ncompilation.\n\nAt least two arguments are expected:\n\n  1. The first argument is a tensor template of the data to\n     be streamed in\n\n  2. The second argument is a tensor with the stream initial state\n\nThe streaming function must return a two element tuple, the\nfirst element is the data to be sent and the second is the\naccumulator.\n\nFor each streamed chunk, you must call `Nx.Stream.send/2` and\n`Nx.Stream.recv/1`. You don't need to call `recv` immediately\nafter `send`, but doing so can be a useful mechanism to provide\nbackpressure. Once all chunks are sent, you must use `Nx.Stream.done/1`\nto receive the accumulated result. Let's see an example:\n\n    defmodule Streamed do\n      import Nx.Defn\n\n      defn sum(tensor, acc) do\n        {acc, tensor + acc}\n      end\n    end\n\nNow let's invoke it:\n\n    stream = EXLA.stream(&Streamed.sum/2, [Nx.template({}, {:s, 64}), 0])\n\n    for i <- 1..5 do\n      Nx.Stream.send(stream, i)\n      IO.inspect {:chunk, Nx.Stream.recv(stream)}\n    end\n\n    IO.inspect {:result, Nx.Stream.done(stream)}\n\nIt will print:\n\n    {:chunk, 0}\n    {:chunk, 1}\n    {:chunk, 2}\n    {:chunk, 3}\n    {:chunk, 4}\n    {:result, 5}\n\n**Note:** While any process can call `Nx.Stream.send/2`, EXLA\nexpects the process that starts the streaming to be the one\ncalling `Nx.Stream.recv/1` and `Nx.Stream.done/1`.\n\nSee `jit/2` for supported options.","ref":"EXLA.html#stream/3"},{"type":"function","title":"EXLA.stream_cached?/3","doc":"Checks if the JIT compilation of stream with\nargs is cached.\n\nNote that hooks are part of the cache, and\ntherefore they must be included in the options.","ref":"EXLA.html#stream_cached?/3"},{"type":"function","title":"Examples - EXLA.stream_cached?/3","doc":"iex> left = Nx.tensor(1, type: {:u, 8})\n    iex> right = Nx.tensor([1, 2, 3], type: {:u, 16})\n    iex> fun = fn x, acc -> {acc, Nx.add(x, acc)} end\n    iex> stream = EXLA.stream(fun, [left, right])\n    iex> Nx.Stream.done(stream)\n    iex> EXLA.stream_cached?(fun, [left, right])\n    true\n    iex> EXLA.stream_cached?(fun, [left, Nx.tensor([1, 2, 3, 4], type: {:u, 16})])\n    false","ref":"EXLA.html#stream_cached?/3-examples"},{"type":"module","title":"EXLA.Backend","doc":"A Nx tensor backend for the data kept on the device.\n\nYou can directly transfer to this backend by calling\n`Nx.backend_transfer/2` or `Nx.backend_copy/2`. It\nallows the following options:\n\n  * `:client` - the client to store the data on.\n    Defaults to EXLA's default client.\n\n  * `:device_id` - which device to store it on.\n\nTo get the data out of the device backend into a regular\ntensor, call `Nx.backend_transfer/1` (with the device\ntensor as the single argument).\n\nNote that the `EXLA.Backend` is asynchronous: operations\non its tensors *may* return immediately, before the tensor\ndata is available. The backend will then block only when\ntrying to read the data or when passing it to another operation.","ref":"EXLA.Backend.html"},{"type":"module","title":"EXLA.MLIR.Function","doc":"Representation of an MLIR Function or `func.func` type.","ref":"EXLA.MLIR.Function.html"},{"type":"function","title":"EXLA.MLIR.Function.get_arguments/1","doc":"Returns a list of references to the function's positional arguments\nwhich can be used in MLIR operations.","ref":"EXLA.MLIR.Function.html#get_arguments/1"},{"type":"function","title":"EXLA.MLIR.Function.pop_region/1","doc":"","ref":"EXLA.MLIR.Function.html#pop_region/1"},{"type":"function","title":"EXLA.MLIR.Function.push_region/2","doc":"","ref":"EXLA.MLIR.Function.html#push_region/2"},{"type":"module","title":"EXLA.MLIR.Module","doc":"Representation of an MLIR module.","ref":"EXLA.MLIR.Module.html"},{"type":"function","title":"EXLA.MLIR.Module.add_function/4","doc":"Adds a new function to the MLIR module.","ref":"EXLA.MLIR.Module.html#add_function/4"},{"type":"function","title":"EXLA.MLIR.Module.compile/5","doc":"Compiles a module into an executable.","ref":"EXLA.MLIR.Module.html#compile/5"},{"type":"function","title":"Options - EXLA.MLIR.Module.compile/5","doc":"* `:device_id` - the device id to compile to and run the executable on.\n    Defaults to the `:default_device_id` on the client. If `:num_replicas`\n    or `:num_partitions` are given, this option is ignored and the device\n    id is set to `-1`.\n\n  * `:num_replicas` - the number of replicas this computation will run on.\n    It defaults to 1 but you can set it if you want to enable single-program\n    multiple data.\n\n  * `:use_spmd` - enables Single-Program Multiple-Data partioning.\n    This is set to true if `:num_partitions` is more than one, otherwise is `false`.\n\nCurrently those options do not have an effect as they related to running the\nsame compiled executable on multiple replicas.\n\nSome options apply to TPU only:\n\n  * `:num_partitions` - the number of partitions this computation will run on.","ref":"EXLA.MLIR.Module.html#compile/5-options"},{"type":"function","title":"EXLA.MLIR.Module.new/3","doc":"Creates a new MLIR module.","ref":"EXLA.MLIR.Module.html#new/3"},{"type":"module","title":"EXLA.MLIR.Value","doc":"Representation of an MLIR Value.\n\nMLIR Values are SSA and generally are either operations or\nblock arguments. This module is used to construct most of the\nMLIR operations.","ref":"EXLA.MLIR.Value.html"},{"type":"function","title":"EXLA.MLIR.Value.abs/1","doc":"","ref":"EXLA.MLIR.Value.html#abs/1"},{"type":"function","title":"EXLA.MLIR.Value.acos/1","doc":"","ref":"EXLA.MLIR.Value.html#acos/1"},{"type":"function","title":"EXLA.MLIR.Value.acosh/1","doc":"","ref":"EXLA.MLIR.Value.html#acosh/1"},{"type":"function","title":"EXLA.MLIR.Value.add/3","doc":"","ref":"EXLA.MLIR.Value.html#add/3"},{"type":"function","title":"EXLA.MLIR.Value.asin/1","doc":"","ref":"EXLA.MLIR.Value.html#asin/1"},{"type":"function","title":"EXLA.MLIR.Value.asinh/1","doc":"","ref":"EXLA.MLIR.Value.html#asinh/1"},{"type":"function","title":"EXLA.MLIR.Value.atan2/3","doc":"","ref":"EXLA.MLIR.Value.html#atan2/3"},{"type":"function","title":"EXLA.MLIR.Value.atan/1","doc":"","ref":"EXLA.MLIR.Value.html#atan/1"},{"type":"function","title":"EXLA.MLIR.Value.atanh/1","doc":"","ref":"EXLA.MLIR.Value.html#atanh/1"},{"type":"function","title":"EXLA.MLIR.Value.bitcast_convert/2","doc":"","ref":"EXLA.MLIR.Value.html#bitcast_convert/2"},{"type":"function","title":"EXLA.MLIR.Value.bitwise_and/3","doc":"","ref":"EXLA.MLIR.Value.html#bitwise_and/3"},{"type":"function","title":"EXLA.MLIR.Value.bitwise_not/1","doc":"","ref":"EXLA.MLIR.Value.html#bitwise_not/1"},{"type":"function","title":"EXLA.MLIR.Value.bitwise_or/3","doc":"","ref":"EXLA.MLIR.Value.html#bitwise_or/3"},{"type":"function","title":"EXLA.MLIR.Value.bitwise_xor/3","doc":"","ref":"EXLA.MLIR.Value.html#bitwise_xor/3"},{"type":"function","title":"EXLA.MLIR.Value.broadcast_in_dim/3","doc":"","ref":"EXLA.MLIR.Value.html#broadcast_in_dim/3"},{"type":"function","title":"EXLA.MLIR.Value.call/3","doc":"","ref":"EXLA.MLIR.Value.html#call/3"},{"type":"function","title":"EXLA.MLIR.Value.cbrt/1","doc":"","ref":"EXLA.MLIR.Value.html#cbrt/1"},{"type":"function","title":"EXLA.MLIR.Value.ceil/1","doc":"","ref":"EXLA.MLIR.Value.html#ceil/1"},{"type":"function","title":"EXLA.MLIR.Value.clamp/3","doc":"","ref":"EXLA.MLIR.Value.html#clamp/3"},{"type":"function","title":"EXLA.MLIR.Value.concatenate/2","doc":"","ref":"EXLA.MLIR.Value.html#concatenate/2"},{"type":"function","title":"EXLA.MLIR.Value.conjugate/1","doc":"","ref":"EXLA.MLIR.Value.html#conjugate/1"},{"type":"function","title":"EXLA.MLIR.Value.constant_from_binary/3","doc":"","ref":"EXLA.MLIR.Value.html#constant_from_binary/3"},{"type":"function","title":"EXLA.MLIR.Value.constant_r0/3","doc":"","ref":"EXLA.MLIR.Value.html#constant_r0/3"},{"type":"function","title":"EXLA.MLIR.Value.convert/2","doc":"","ref":"EXLA.MLIR.Value.html#convert/2"},{"type":"function","title":"EXLA.MLIR.Value.convolution/11","doc":"","ref":"EXLA.MLIR.Value.html#convolution/11"},{"type":"function","title":"EXLA.MLIR.Value.cos/1","doc":"","ref":"EXLA.MLIR.Value.html#cos/1"},{"type":"function","title":"EXLA.MLIR.Value.cosh/1","doc":"","ref":"EXLA.MLIR.Value.html#cosh/1"},{"type":"function","title":"EXLA.MLIR.Value.count_leading_zeros/1","doc":"","ref":"EXLA.MLIR.Value.html#count_leading_zeros/1"},{"type":"function","title":"EXLA.MLIR.Value.create_token/1","doc":"","ref":"EXLA.MLIR.Value.html#create_token/1"},{"type":"function","title":"EXLA.MLIR.Value.divide/3","doc":"","ref":"EXLA.MLIR.Value.html#divide/3"},{"type":"function","title":"EXLA.MLIR.Value.dot_general/5","doc":"","ref":"EXLA.MLIR.Value.html#dot_general/5"},{"type":"function","title":"EXLA.MLIR.Value.dynamic_slice/3","doc":"","ref":"EXLA.MLIR.Value.html#dynamic_slice/3"},{"type":"function","title":"EXLA.MLIR.Value.dynamic_update_slice/3","doc":"","ref":"EXLA.MLIR.Value.html#dynamic_update_slice/3"},{"type":"function","title":"EXLA.MLIR.Value.equal/3","doc":"","ref":"EXLA.MLIR.Value.html#equal/3"},{"type":"function","title":"EXLA.MLIR.Value.erf/1","doc":"","ref":"EXLA.MLIR.Value.html#erf/1"},{"type":"function","title":"EXLA.MLIR.Value.erf_inv/1","doc":"","ref":"EXLA.MLIR.Value.html#erf_inv/1"},{"type":"function","title":"EXLA.MLIR.Value.erfc/1","doc":"","ref":"EXLA.MLIR.Value.html#erfc/1"},{"type":"function","title":"EXLA.MLIR.Value.exp/1","doc":"","ref":"EXLA.MLIR.Value.html#exp/1"},{"type":"function","title":"EXLA.MLIR.Value.expm1/1","doc":"","ref":"EXLA.MLIR.Value.html#expm1/1"},{"type":"function","title":"EXLA.MLIR.Value.fft/3","doc":"","ref":"EXLA.MLIR.Value.html#fft/3"},{"type":"function","title":"EXLA.MLIR.Value.floor/1","doc":"","ref":"EXLA.MLIR.Value.html#floor/1"},{"type":"function","title":"EXLA.MLIR.Value.gather/7","doc":"","ref":"EXLA.MLIR.Value.html#gather/7"},{"type":"function","title":"EXLA.MLIR.Value.get_shape/1","doc":"","ref":"EXLA.MLIR.Value.html#get_shape/1"},{"type":"function","title":"EXLA.MLIR.Value.get_tuple_element/2","doc":"","ref":"EXLA.MLIR.Value.html#get_tuple_element/2"},{"type":"function","title":"EXLA.MLIR.Value.greater/3","doc":"","ref":"EXLA.MLIR.Value.html#greater/3"},{"type":"function","title":"EXLA.MLIR.Value.greater_equal/3","doc":"","ref":"EXLA.MLIR.Value.html#greater_equal/3"},{"type":"function","title":"EXLA.MLIR.Value.if_op/2","doc":"","ref":"EXLA.MLIR.Value.html#if_op/2"},{"type":"function","title":"EXLA.MLIR.Value.imag/1","doc":"","ref":"EXLA.MLIR.Value.html#imag/1"},{"type":"function","title":"EXLA.MLIR.Value.infeed/2","doc":"","ref":"EXLA.MLIR.Value.html#infeed/2"},{"type":"function","title":"EXLA.MLIR.Value.iota/3","doc":"","ref":"EXLA.MLIR.Value.html#iota/3"},{"type":"function","title":"EXLA.MLIR.Value.is_infinity/1","doc":"","ref":"EXLA.MLIR.Value.html#is_infinity/1"},{"type":"function","title":"EXLA.MLIR.Value.is_nan/1","doc":"","ref":"EXLA.MLIR.Value.html#is_nan/1"},{"type":"function","title":"EXLA.MLIR.Value.left_shift/3","doc":"","ref":"EXLA.MLIR.Value.html#left_shift/3"},{"type":"function","title":"EXLA.MLIR.Value.less/3","doc":"","ref":"EXLA.MLIR.Value.html#less/3"},{"type":"function","title":"EXLA.MLIR.Value.less_equal/3","doc":"","ref":"EXLA.MLIR.Value.html#less_equal/3"},{"type":"function","title":"EXLA.MLIR.Value.log1p/1","doc":"","ref":"EXLA.MLIR.Value.html#log1p/1"},{"type":"function","title":"EXLA.MLIR.Value.log/1","doc":"","ref":"EXLA.MLIR.Value.html#log/1"},{"type":"function","title":"EXLA.MLIR.Value.map/3","doc":"","ref":"EXLA.MLIR.Value.html#map/3"},{"type":"function","title":"EXLA.MLIR.Value.max/3","doc":"","ref":"EXLA.MLIR.Value.html#max/3"},{"type":"function","title":"EXLA.MLIR.Value.min/3","doc":"","ref":"EXLA.MLIR.Value.html#min/3"},{"type":"function","title":"EXLA.MLIR.Value.multiply/3","doc":"","ref":"EXLA.MLIR.Value.html#multiply/3"},{"type":"function","title":"EXLA.MLIR.Value.negate/1","doc":"","ref":"EXLA.MLIR.Value.html#negate/1"},{"type":"function","title":"EXLA.MLIR.Value.not_equal/3","doc":"","ref":"EXLA.MLIR.Value.html#not_equal/3"},{"type":"function","title":"EXLA.MLIR.Value.optimization_barrier/1","doc":"","ref":"EXLA.MLIR.Value.html#optimization_barrier/1"},{"type":"function","title":"EXLA.MLIR.Value.outfeed/2","doc":"","ref":"EXLA.MLIR.Value.html#outfeed/2"},{"type":"function","title":"EXLA.MLIR.Value.pad/3","doc":"","ref":"EXLA.MLIR.Value.html#pad/3"},{"type":"function","title":"EXLA.MLIR.Value.population_count/1","doc":"","ref":"EXLA.MLIR.Value.html#population_count/1"},{"type":"function","title":"EXLA.MLIR.Value.pow/3","doc":"","ref":"EXLA.MLIR.Value.html#pow/3"},{"type":"function","title":"EXLA.MLIR.Value.qr/3","doc":"","ref":"EXLA.MLIR.Value.html#qr/3"},{"type":"function","title":"EXLA.MLIR.Value.real/1","doc":"","ref":"EXLA.MLIR.Value.html#real/1"},{"type":"function","title":"EXLA.MLIR.Value.reduce/4","doc":"","ref":"EXLA.MLIR.Value.html#reduce/4"},{"type":"function","title":"EXLA.MLIR.Value.remainder/3","doc":"","ref":"EXLA.MLIR.Value.html#remainder/3"},{"type":"function","title":"EXLA.MLIR.Value.reshape/2","doc":"","ref":"EXLA.MLIR.Value.html#reshape/2"},{"type":"function","title":"EXLA.MLIR.Value.reverse/2","doc":"","ref":"EXLA.MLIR.Value.html#reverse/2"},{"type":"function","title":"EXLA.MLIR.Value.right_shift_arithmetic/3","doc":"","ref":"EXLA.MLIR.Value.html#right_shift_arithmetic/3"},{"type":"function","title":"EXLA.MLIR.Value.right_shift_logical/3","doc":"","ref":"EXLA.MLIR.Value.html#right_shift_logical/3"},{"type":"function","title":"EXLA.MLIR.Value.round/1","doc":"","ref":"EXLA.MLIR.Value.html#round/1"},{"type":"function","title":"EXLA.MLIR.Value.rsqrt/1","doc":"","ref":"EXLA.MLIR.Value.html#rsqrt/1"},{"type":"function","title":"EXLA.MLIR.Value.scatter/8","doc":"","ref":"EXLA.MLIR.Value.html#scatter/8"},{"type":"function","title":"EXLA.MLIR.Value.select/3","doc":"","ref":"EXLA.MLIR.Value.html#select/3"},{"type":"function","title":"EXLA.MLIR.Value.select_and_scatter/7","doc":"","ref":"EXLA.MLIR.Value.html#select_and_scatter/7"},{"type":"function","title":"EXLA.MLIR.Value.sigmoid/1","doc":"","ref":"EXLA.MLIR.Value.html#sigmoid/1"},{"type":"function","title":"EXLA.MLIR.Value.sign/1","doc":"","ref":"EXLA.MLIR.Value.html#sign/1"},{"type":"function","title":"EXLA.MLIR.Value.sin/1","doc":"","ref":"EXLA.MLIR.Value.html#sin/1"},{"type":"function","title":"EXLA.MLIR.Value.sinh/1","doc":"","ref":"EXLA.MLIR.Value.html#sinh/1"},{"type":"function","title":"EXLA.MLIR.Value.slice/4","doc":"","ref":"EXLA.MLIR.Value.html#slice/4"},{"type":"function","title":"EXLA.MLIR.Value.sort/4","doc":"","ref":"EXLA.MLIR.Value.html#sort/4"},{"type":"function","title":"EXLA.MLIR.Value.sqrt/1","doc":"","ref":"EXLA.MLIR.Value.html#sqrt/1"},{"type":"function","title":"EXLA.MLIR.Value.subtract/3","doc":"","ref":"EXLA.MLIR.Value.html#subtract/3"},{"type":"function","title":"EXLA.MLIR.Value.tan/1","doc":"","ref":"EXLA.MLIR.Value.html#tan/1"},{"type":"function","title":"EXLA.MLIR.Value.tanh/1","doc":"","ref":"EXLA.MLIR.Value.html#tanh/1"},{"type":"function","title":"EXLA.MLIR.Value.top_k/2","doc":"","ref":"EXLA.MLIR.Value.html#top_k/2"},{"type":"function","title":"EXLA.MLIR.Value.transpose/2","doc":"","ref":"EXLA.MLIR.Value.html#transpose/2"},{"type":"function","title":"EXLA.MLIR.Value.triangular_solve/5","doc":"","ref":"EXLA.MLIR.Value.html#triangular_solve/5"},{"type":"function","title":"EXLA.MLIR.Value.tuple/2","doc":"","ref":"EXLA.MLIR.Value.html#tuple/2"},{"type":"function","title":"EXLA.MLIR.Value.variadic_return/2","doc":"","ref":"EXLA.MLIR.Value.html#variadic_return/2"},{"type":"function","title":"EXLA.MLIR.Value.while/2","doc":"","ref":"EXLA.MLIR.Value.html#while/2"},{"type":"function","title":"EXLA.MLIR.Value.window_reduce/8","doc":"","ref":"EXLA.MLIR.Value.html#window_reduce/8"},{"type":"module","title":"EXLA.BinaryBuffer","doc":"A buffer where data is kept in a binary.","ref":"EXLA.BinaryBuffer.html"},{"type":"function","title":"EXLA.BinaryBuffer.from_binary/2","doc":"","ref":"EXLA.BinaryBuffer.html#from_binary/2"},{"type":"module","title":"EXLA.Builder","doc":"Wrapper around XLA's builder.","ref":"EXLA.Builder.html"},{"type":"function","title":"EXLA.Builder.build/1","doc":"","ref":"EXLA.Builder.html#build/1"},{"type":"function","title":"EXLA.Builder.exla_shape/2","doc":"","ref":"EXLA.Builder.html#exla_shape/2"},{"type":"function","title":"EXLA.Builder.new/1","doc":"","ref":"EXLA.Builder.html#new/1"},{"type":"function","title":"EXLA.Builder.new/2","doc":"","ref":"EXLA.Builder.html#new/2"},{"type":"module","title":"EXLA.Client","doc":"Functions for managing `EXLA.Client`.\n\nSee `EXLA` module docs for a general introduction.","ref":"EXLA.Client.html"},{"type":"function","title":"EXLA.Client.default_name/0","doc":"Returns the name of the default client.","ref":"EXLA.Client.html#default_name/0"},{"type":"function","title":"EXLA.Client.fetch!/1","doc":"Fetches a client with the given `name` from configuration.","ref":"EXLA.Client.html#fetch!/1"},{"type":"function","title":"EXLA.Client.from_outfeed/5","doc":"Sends buffer from device outfeed to the given process tagged by `ref`.\n\n> Note: XLA does not support tuple outfeed shapes. Passing one will simply\n> block the operation indefinitely. Instead, convert the tuple into multiple\n> outfeed operations.","ref":"EXLA.Client.html#from_outfeed/5"},{"type":"function","title":"EXLA.Client.get_supported_platforms/0","doc":"Returns a map of supported platforms with device information.","ref":"EXLA.Client.html#get_supported_platforms/0"},{"type":"function","title":"EXLA.Client.to_infeed/3","doc":"Sends `data_and_shapes` to device infeed.\n\n`data_and_shapes` must be a list of two element tuples where the\nfirst element is a binary or a flat list of binaries and the second\nelement is a `EXLA.Shape`.\n\n> Note: XLA does not support tuple infeed shapes when running on\n> host. Passing one will simply block the operation indefinitely.\n> Instead, convert the tuple into multiple infeed operations.","ref":"EXLA.Client.html#to_infeed/3"},{"type":"module","title":"EXLA.Computation","doc":"Wrapper around XLA's computation.","ref":"EXLA.Computation.html"},{"type":"function","title":"EXLA.Computation.compile/4","doc":"Compiles a computation into an executable.","ref":"EXLA.Computation.html#compile/4"},{"type":"function","title":"Options - EXLA.Computation.compile/4","doc":"* `:device_id` - the device id to compile to and run the executable on.\n    Defaults to the `:default_device_id` on the client. If `:num_replicas`\n    or `:num_partitions` are given, this option is ignored and the device\n    id is set to `-1`.\n\n  * `:num_replicas` - the number of replicas this computation will run on.\n    It defaults to 1 but you can set it if you want to enable single-program\n    multiple data.\n\n  * `:use_spmd` - enables Single-Program Multiple-Data partioning.\n    This is set to true if `:num_partitions` is more than one, otherwise is `false`.\n\nCurrently those options do not have an effect as they related to running the\nsame compiled executable on multiple replicas.\n\nSome options apply to TPU only:\n\n  * `:num_partitions` - the number of partitions this computation will run on.","ref":"EXLA.Computation.html#compile/4-options"},{"type":"module","title":"EXLA.DeviceBuffer","doc":"An EXLA DeviceBuffer for data allocated in the device.","ref":"EXLA.DeviceBuffer.html"},{"type":"function","title":"EXLA.DeviceBuffer.copy_to_device/3","doc":"Copies buffer to device with given device ID.","ref":"EXLA.DeviceBuffer.html#copy_to_device/3"},{"type":"function","title":"EXLA.DeviceBuffer.deallocate/1","doc":"Deallocates the underlying buffer.\n\nReturns `:ok` | `:already_deallocated`.","ref":"EXLA.DeviceBuffer.html#deallocate/1"},{"type":"function","title":"EXLA.DeviceBuffer.place_on_device/4","doc":"Places the given binary `buffer` on the given `device` using `client`.","ref":"EXLA.DeviceBuffer.html#place_on_device/4"},{"type":"function","title":"EXLA.DeviceBuffer.read/2","doc":"Reads `size` from the underlying buffer ref.\n\nThis copies the underlying device memory into a binary\nwithout destroying it. If `size` is negative, then it\nreads the whole buffer.","ref":"EXLA.DeviceBuffer.html#read/2"},{"type":"module","title":"EXLA.Executable","doc":"Wrapper around XLA's executable.","ref":"EXLA.Executable.html"},{"type":"function","title":"EXLA.Executable.deserialize/2","doc":"","ref":"EXLA.Executable.html#deserialize/2"},{"type":"function","title":"EXLA.Executable.run/3","doc":"Runs the given executable with a list of lists as inputs and the given options.","ref":"EXLA.Executable.html#run/3"},{"type":"function","title":"EXLA.Executable.serialize/1","doc":"","ref":"EXLA.Executable.html#serialize/1"},{"type":"module","title":"EXLA.Lib","doc":"High-level operations built on top of `EXLA.Op`.","ref":"EXLA.Lib.html"},{"type":"function","title":"EXLA.Lib.argmax/4","doc":"Computes the argmax of the given operation.","ref":"EXLA.Lib.html#argmax/4"},{"type":"function","title":"Options - EXLA.Lib.argmax/4","doc":"* `:axis` - the axis to reduce on\n  * `:keep_axis` - whether or not to keep reduced axis\n  * `:tie_break` - how to break ties","ref":"EXLA.Lib.html#argmax/4-options"},{"type":"function","title":"EXLA.Lib.argmin/4","doc":"Computes the argmin of the given operation.","ref":"EXLA.Lib.html#argmin/4"},{"type":"function","title":"Options - EXLA.Lib.argmin/4","doc":"* `:axis` - the axis to reduce on\n  * `:keep_axis` - whether or not to keep reduced axis\n  * `:tie_break` - how to break ties","ref":"EXLA.Lib.html#argmin/4-options"},{"type":"function","title":"EXLA.Lib.argsort/6","doc":"Sorts a tensor and returns the corresponding indices in the new positions.","ref":"EXLA.Lib.html#argsort/6"},{"type":"function","title":"EXLA.Lib.iota/3","doc":"Builds iota along axis.","ref":"EXLA.Lib.html#iota/3"},{"type":"function","title":"EXLA.Lib.max_number/2","doc":"Returns a maximum value scalar operator for the given type.\n\nMaximum values are defined in `Nx.Type.max_finite_binary/1`.","ref":"EXLA.Lib.html#max_number/2"},{"type":"function","title":"EXLA.Lib.min_number/2","doc":"Returns a minimum value scalar operator for the given type.\n\nIt will be negative infinity for floating point types.","ref":"EXLA.Lib.html#min_number/2"},{"type":"function","title":"EXLA.Lib.tan/1","doc":"Element-wise tangent function.","ref":"EXLA.Lib.html#tan/1"},{"type":"module","title":"EXLA.Op","doc":"Wrapper around XLA's ops.","ref":"EXLA.Op.html"},{"type":"function","title":"EXLA.Op.abs/1","doc":"Unary abs.","ref":"EXLA.Op.html#abs/1"},{"type":"function","title":"EXLA.Op.acos/1","doc":"Unary acos.","ref":"EXLA.Op.html#acos/1"},{"type":"function","title":"EXLA.Op.acosh/1","doc":"Unary acosh.","ref":"EXLA.Op.html#acosh/1"},{"type":"function","title":"EXLA.Op.add/3","doc":"Element-wise add with broadcasting.","ref":"EXLA.Op.html#add/3"},{"type":"function","title":"EXLA.Op.asin/1","doc":"Unary asin.","ref":"EXLA.Op.html#asin/1"},{"type":"function","title":"EXLA.Op.asinh/1","doc":"Unary asinh.","ref":"EXLA.Op.html#asinh/1"},{"type":"function","title":"EXLA.Op.atan2/3","doc":"Element-wise atan2 with broadcasting.","ref":"EXLA.Op.html#atan2/3"},{"type":"function","title":"EXLA.Op.atan/1","doc":"Unary atan.","ref":"EXLA.Op.html#atan/1"},{"type":"function","title":"EXLA.Op.atanh/1","doc":"Unary atanh.","ref":"EXLA.Op.html#atanh/1"},{"type":"function","title":"EXLA.Op.bitcast_convert_type/2","doc":"","ref":"EXLA.Op.html#bitcast_convert_type/2"},{"type":"function","title":"EXLA.Op.bitwise_and/3","doc":"Element-wise bitwise_and with broadcasting.","ref":"EXLA.Op.html#bitwise_and/3"},{"type":"function","title":"EXLA.Op.bitwise_not/1","doc":"Unary bitwise_not.","ref":"EXLA.Op.html#bitwise_not/1"},{"type":"function","title":"EXLA.Op.bitwise_or/3","doc":"Element-wise bitwise_or with broadcasting.","ref":"EXLA.Op.html#bitwise_or/3"},{"type":"function","title":"EXLA.Op.bitwise_xor/3","doc":"Element-wise bitwise_xor with broadcasting.","ref":"EXLA.Op.html#bitwise_xor/3"},{"type":"function","title":"EXLA.Op.broadcast_in_dim/3","doc":"Broadcasts the tensor to `shape`.","ref":"EXLA.Op.html#broadcast_in_dim/3"},{"type":"function","title":"EXLA.Op.call/3","doc":"","ref":"EXLA.Op.html#call/3"},{"type":"function","title":"EXLA.Op.cbrt/1","doc":"Unary cbrt.","ref":"EXLA.Op.html#cbrt/1"},{"type":"function","title":"EXLA.Op.ceil/1","doc":"Unary ceil.","ref":"EXLA.Op.html#ceil/1"},{"type":"function","title":"EXLA.Op.clamp/3","doc":"","ref":"EXLA.Op.html#clamp/3"},{"type":"function","title":"EXLA.Op.concatenate/2","doc":"","ref":"EXLA.Op.html#concatenate/2"},{"type":"function","title":"EXLA.Op.conditional/3","doc":"","ref":"EXLA.Op.html#conditional/3"},{"type":"function","title":"EXLA.Op.conditional/5","doc":"","ref":"EXLA.Op.html#conditional/5"},{"type":"function","title":"EXLA.Op.conjugate/1","doc":"","ref":"EXLA.Op.html#conjugate/1"},{"type":"function","title":"EXLA.Op.constant_from_binary/3","doc":"Creates a n-dimensional constant from binary `data` with `shape`.","ref":"EXLA.Op.html#constant_from_binary/3"},{"type":"function","title":"EXLA.Op.constant_r0/3","doc":"Creates a numeric constant.","ref":"EXLA.Op.html#constant_r0/3"},{"type":"function","title":"EXLA.Op.conv_general_dilated/10","doc":"","ref":"EXLA.Op.html#conv_general_dilated/10"},{"type":"function","title":"EXLA.Op.convert_element_type/2","doc":"","ref":"EXLA.Op.html#convert_element_type/2"},{"type":"function","title":"EXLA.Op.cos/1","doc":"Unary cos.","ref":"EXLA.Op.html#cos/1"},{"type":"function","title":"EXLA.Op.cosh/1","doc":"Unary cosh.","ref":"EXLA.Op.html#cosh/1"},{"type":"function","title":"EXLA.Op.count_leading_zeros/1","doc":"Unary count_leading_zeros.","ref":"EXLA.Op.html#count_leading_zeros/1"},{"type":"function","title":"EXLA.Op.create_token/1","doc":"","ref":"EXLA.Op.html#create_token/1"},{"type":"function","title":"EXLA.Op.divide/3","doc":"Element-wise divide with broadcasting.","ref":"EXLA.Op.html#divide/3"},{"type":"function","title":"EXLA.Op.dot/3","doc":"","ref":"EXLA.Op.html#dot/3"},{"type":"function","title":"EXLA.Op.dot_general/4","doc":"","ref":"EXLA.Op.html#dot_general/4"},{"type":"function","title":"EXLA.Op.dynamic_slice/3","doc":"","ref":"EXLA.Op.html#dynamic_slice/3"},{"type":"function","title":"EXLA.Op.dynamic_update_slice/3","doc":"","ref":"EXLA.Op.html#dynamic_update_slice/3"},{"type":"function","title":"EXLA.Op.equal/3","doc":"Element-wise equal with broadcasting.","ref":"EXLA.Op.html#equal/3"},{"type":"function","title":"EXLA.Op.erf/1","doc":"Unary erf.","ref":"EXLA.Op.html#erf/1"},{"type":"function","title":"EXLA.Op.erf_inv/1","doc":"Unary erf_inv.","ref":"EXLA.Op.html#erf_inv/1"},{"type":"function","title":"EXLA.Op.erfc/1","doc":"Unary erfc.","ref":"EXLA.Op.html#erfc/1"},{"type":"function","title":"EXLA.Op.exp/1","doc":"Unary exp.","ref":"EXLA.Op.html#exp/1"},{"type":"function","title":"EXLA.Op.expm1/1","doc":"Unary expm1.","ref":"EXLA.Op.html#expm1/1"},{"type":"function","title":"EXLA.Op.fft/2","doc":"","ref":"EXLA.Op.html#fft/2"},{"type":"function","title":"EXLA.Op.floor/1","doc":"Unary floor.","ref":"EXLA.Op.html#floor/1"},{"type":"function","title":"EXLA.Op.gather/7","doc":"The XLA gather operation stitches together several slices\nof an input array.\n\nNote that this operation is extremely generic and far from\nintuitive for regular usage. However, it can be used to implement\nmany specific operations that have to do with combining multiple\ntensor slices.","ref":"EXLA.Op.html#gather/7"},{"type":"function","title":"Parameteres - EXLA.Op.gather/7","doc":"The XLA docs are rather cryptic unless already understood,\nso here's an attempt of a more intuitive description.\n\n### `index_vector_dim`\n\nDetermines which dimension contains index vectors. In most cases\nwe want to set this to the last dimension.\n\n    given\n      start_indices = [[0, 1], [1, 1]]\n    and given\n      index_vector_dim = 1\n    then\n      index vectors are [0, 1] and [1, 1]\n\nNote that we can set this to `last_dimension + 1`, in which case\n`start_indices` are implicitly reshaped to have a trailing dimension\nof 1.\n\n    given\n      start_indices = [[0, 1], [1, 1]]\n    and given\n      index_vector_dim = 2\n    then\n      start_indices <- [[[0], [1]], [[1], [1]]]\n      index vectors are [0], [1], [1], [1]\n\n### `start_index_map`\n\nNote: though given as a list, it can be treated as a map of `list_idx -> value`.\n\nAn index vector may have less elements than the operand tensor shape.\nFor example:\n\n    given\n      operand = [[1, 2], [3, 4]]\n      start_indices = [[1], [0]]\n      index_vector_dim = 1\n\nAs described above, in this case index vectors are `[1]`, `[0]` and they have\nlength 1. However, the operand has rank 2, so we need vectors of the form `[_, _]`\nto point to a specific element in the operand. The `start_index_map` determines\nwhere indices go into this template:\n\n    and given\n      start_index_map = [0] # effectively %{0 => 0}\n    then\n      actual index vectors are [1, _] and [0, _]\n\n    and given\n      start_index_map = [1] # effectively %{0 => 1}\n    then\n      actual index vectors are [_, 1] and [_, 0]\n\nFinally, the missing elements (`_`) are assumed to be 0.\n\nComplete examples:\n\n    given\n      operand = [[1, 2], [3, 4]]\n      start_indices = [[0], [1]]\n      index_vector_dim = 1\n    and given\n      start_index_map = [1] # effectively %{0 => 1}\n    then\n      actual index vectors are [0, 0], [0, 1] (leading 0 is inserted)\n\n    given\n      operand = [[1, 2], [3, 4]]\n      start_indices = [[0, 1], [1, 1]]\n      index_vector_dim = 1\n    and given\n      start_index_map = [0, 1] # effectively %{0 => 0, 1 => 1}\n    then\n      actual index vectors are [0, 1], [1, 1] (as expected)\n\n    given\n      operand = [[1, 2], [3, 4]]\n      start_indices = [[0, 1], [1, 1]]\n      index_vector_dim = 1\n    and given\n      start_index_map = [1, 0] # effectively %{0 => 1, 1 => 0}\n    then\n      actual index vectors are [1, 0], [1, 1] (see how the first vector is reversed)\n\n### `slice_sizes`\n\nFor every starting point (as described above) we take a slice given\nby `slice_sizes`. Naturally, `slice_sizes` must have the same length\nas operand rank, so that we have one size per dimension.\n\n    given\n      operand = [[1, 2], [3, 4]]\n      actual index vector [1, 0]\n    and given\n      slice_sizes = [1, 2]\n    then\n      slice for actual index vector is [[3, 4]]\n\n### `collapsed_slice_dims`\n\nA list of dimensions that are collapsed (effectively removed) in\nthe slice shape. Only dimensions of size 1 can be collapsed.\n\n    given\n      slice is [[3, 4]] # shape: [1][2]\n    and given\n      collapsed_slice_dims = [0]\n    then\n      actual slice is [3, 4] # shape [2]\n\n### `offset_dims`\n\nA list of dimensions in the output tensor corresponding to the\nnon-collapsed dimensions in slice tensors. In other words, these\ndimensions are used for indexing elements of the slice tensors.\n\n    given\n      operand = [[1, 2], [3, 4]]\n      start_indices = [[1, 0], [0, 0], [1, 0]]\n      index_vector_dim = 1\n      start_index_map = [1, 2] # effectively %{0 => 0, 1 => 1}\n      collapsed_slice_dims = [0]\n    and given\n      offset_dims = [1]\n    then\n      result is [[3, 4], [1, 2], [3, 4]]\n\nIn the above example the collapsed slices are `[3, 4]`, `[1, 2]`, `[3, 4]`\nand have rank 1. Using `offset_dims` we specify that the first\ndimension in each slice corresponds to the second dimension in\nthe output tensor.\n\nIf we use the first output dimension instead, we get:\n\n    and given\n      offset_dims = [0]\n    then\n      result is [[3, 1, 3], [4, 2, 4]]","ref":"EXLA.Op.html#gather/7-parameteres"},{"type":"function","title":"Docs - EXLA.Op.gather/7","doc":"More formal specification can be found in [the XLA Gather docs](https://www.tensorflow.org/xla/operation_semantics#gather).","ref":"EXLA.Op.html#gather/7-docs"},{"type":"function","title":"EXLA.Op.get_shape/1","doc":"Gets the shape of an operator.","ref":"EXLA.Op.html#get_shape/1"},{"type":"function","title":"EXLA.Op.get_tuple_element/2","doc":"","ref":"EXLA.Op.html#get_tuple_element/2"},{"type":"function","title":"EXLA.Op.greater/3","doc":"Element-wise greater with broadcasting.","ref":"EXLA.Op.html#greater/3"},{"type":"function","title":"EXLA.Op.greater_equal/3","doc":"Element-wise greater_equal with broadcasting.","ref":"EXLA.Op.html#greater_equal/3"},{"type":"function","title":"EXLA.Op.ifft/2","doc":"","ref":"EXLA.Op.html#ifft/2"},{"type":"function","title":"EXLA.Op.imag/1","doc":"","ref":"EXLA.Op.html#imag/1"},{"type":"function","title":"EXLA.Op.infeed/2","doc":"","ref":"EXLA.Op.html#infeed/2"},{"type":"function","title":"EXLA.Op.iota/3","doc":"Creates iota tensor.","ref":"EXLA.Op.html#iota/3"},{"type":"function","title":"EXLA.Op.is_infinity/4","doc":"","ref":"EXLA.Op.html#is_infinity/4"},{"type":"function","title":"EXLA.Op.is_nan/4","doc":"","ref":"EXLA.Op.html#is_nan/4"},{"type":"function","title":"EXLA.Op.left_shift/3","doc":"Element-wise left_shift with broadcasting.","ref":"EXLA.Op.html#left_shift/3"},{"type":"function","title":"EXLA.Op.less/3","doc":"Element-wise less with broadcasting.","ref":"EXLA.Op.html#less/3"},{"type":"function","title":"EXLA.Op.less_equal/3","doc":"Element-wise less_equal with broadcasting.","ref":"EXLA.Op.html#less_equal/3"},{"type":"function","title":"EXLA.Op.log1p/1","doc":"Unary log1p.","ref":"EXLA.Op.html#log1p/1"},{"type":"function","title":"EXLA.Op.log/1","doc":"Unary log.","ref":"EXLA.Op.html#log/1"},{"type":"function","title":"EXLA.Op.lu/1","doc":"","ref":"EXLA.Op.html#lu/1"},{"type":"function","title":"EXLA.Op.map/3","doc":"","ref":"EXLA.Op.html#map/3"},{"type":"function","title":"EXLA.Op.max/3","doc":"Element-wise max with broadcasting.","ref":"EXLA.Op.html#max/3"},{"type":"function","title":"EXLA.Op.min/3","doc":"Element-wise min with broadcasting.","ref":"EXLA.Op.html#min/3"},{"type":"function","title":"EXLA.Op.multiply/3","doc":"Element-wise multiply with broadcasting.","ref":"EXLA.Op.html#multiply/3"},{"type":"function","title":"EXLA.Op.negate/1","doc":"Unary negate.","ref":"EXLA.Op.html#negate/1"},{"type":"function","title":"EXLA.Op.not_equal/3","doc":"Element-wise not_equal with broadcasting.","ref":"EXLA.Op.html#not_equal/3"},{"type":"function","title":"EXLA.Op.outfeed/2","doc":"","ref":"EXLA.Op.html#outfeed/2"},{"type":"function","title":"EXLA.Op.pad/3","doc":"Pads the tensor with value and padding config.","ref":"EXLA.Op.html#pad/3"},{"type":"function","title":"EXLA.Op.parameter/4","doc":"Specifies a parameter at position `i` with `shape` and `name`.","ref":"EXLA.Op.html#parameter/4"},{"type":"function","title":"EXLA.Op.population_count/1","doc":"Unary population_count.","ref":"EXLA.Op.html#population_count/1"},{"type":"function","title":"EXLA.Op.pow/3","doc":"Element-wise pow with broadcasting.","ref":"EXLA.Op.html#pow/3"},{"type":"function","title":"EXLA.Op.real/1","doc":"","ref":"EXLA.Op.html#real/1"},{"type":"function","title":"EXLA.Op.reduce/4","doc":"","ref":"EXLA.Op.html#reduce/4"},{"type":"function","title":"EXLA.Op.remainder/3","doc":"Element-wise remainder with broadcasting.","ref":"EXLA.Op.html#remainder/3"},{"type":"function","title":"EXLA.Op.reshape/2","doc":"Reshapes the tensor to `shape`.","ref":"EXLA.Op.html#reshape/2"},{"type":"function","title":"EXLA.Op.reverse/2","doc":"","ref":"EXLA.Op.html#reverse/2"},{"type":"function","title":"EXLA.Op.right_shift_arithmetic/3","doc":"Element-wise right_shift_arithmetic with broadcasting.","ref":"EXLA.Op.html#right_shift_arithmetic/3"},{"type":"function","title":"EXLA.Op.right_shift_logical/3","doc":"Element-wise right_shift_logical with broadcasting.","ref":"EXLA.Op.html#right_shift_logical/3"},{"type":"function","title":"EXLA.Op.rng_normal/3","doc":"Creates tensor with normal distribution.","ref":"EXLA.Op.html#rng_normal/3"},{"type":"function","title":"EXLA.Op.rng_uniform/3","doc":"Creates tensor with uniform distribution.","ref":"EXLA.Op.html#rng_uniform/3"},{"type":"function","title":"EXLA.Op.round/1","doc":"Unary round.","ref":"EXLA.Op.html#round/1"},{"type":"function","title":"EXLA.Op.rsqrt/1","doc":"Unary rsqrt.","ref":"EXLA.Op.html#rsqrt/1"},{"type":"function","title":"EXLA.Op.scatter/8","doc":"","ref":"EXLA.Op.html#scatter/8"},{"type":"function","title":"EXLA.Op.select/3","doc":"","ref":"EXLA.Op.html#select/3"},{"type":"function","title":"EXLA.Op.select_and_scatter/8","doc":"","ref":"EXLA.Op.html#select_and_scatter/8"},{"type":"function","title":"EXLA.Op.sigmoid/1","doc":"Unary sigmoid.","ref":"EXLA.Op.html#sigmoid/1"},{"type":"function","title":"EXLA.Op.sign/1","doc":"Unary sign.","ref":"EXLA.Op.html#sign/1"},{"type":"function","title":"EXLA.Op.sin/1","doc":"Unary sin.","ref":"EXLA.Op.html#sin/1"},{"type":"function","title":"EXLA.Op.sinh/1","doc":"Unary sinh.","ref":"EXLA.Op.html#sinh/1"},{"type":"function","title":"EXLA.Op.slice/4","doc":"","ref":"EXLA.Op.html#slice/4"},{"type":"function","title":"EXLA.Op.sort/4","doc":"","ref":"EXLA.Op.html#sort/4"},{"type":"function","title":"EXLA.Op.sqrt/1","doc":"Unary sqrt.","ref":"EXLA.Op.html#sqrt/1"},{"type":"function","title":"EXLA.Op.subtract/3","doc":"Element-wise subtract with broadcasting.","ref":"EXLA.Op.html#subtract/3"},{"type":"function","title":"EXLA.Op.tanh/1","doc":"Unary tanh.","ref":"EXLA.Op.html#tanh/1"},{"type":"function","title":"EXLA.Op.top_k/2","doc":"","ref":"EXLA.Op.html#top_k/2"},{"type":"function","title":"EXLA.Op.transpose/2","doc":"","ref":"EXLA.Op.html#transpose/2"},{"type":"function","title":"EXLA.Op.triangular_solve/6","doc":"","ref":"EXLA.Op.html#triangular_solve/6"},{"type":"function","title":"EXLA.Op.tuple/2","doc":"Builds a tuple with the given elements.","ref":"EXLA.Op.html#tuple/2"},{"type":"function","title":"EXLA.Op.variadic_reduce/5","doc":"","ref":"EXLA.Op.html#variadic_reduce/5"},{"type":"function","title":"EXLA.Op.variadic_sort/5","doc":"","ref":"EXLA.Op.html#variadic_sort/5"},{"type":"function","title":"EXLA.Op.while/3","doc":"","ref":"EXLA.Op.html#while/3"},{"type":"function","title":"EXLA.Op.window_reduce/7","doc":"","ref":"EXLA.Op.html#window_reduce/7"},{"type":"module","title":"EXLA.Shape","doc":"Wrapper around XLA's shape.","ref":"EXLA.Shape.html"},{"type":"function","title":"EXLA.Shape.byte_size/1","doc":"Returns the shape size in bytes.","ref":"EXLA.Shape.html#byte_size/1"},{"type":"function","title":"EXLA.Shape.charlist_to_dtype/1","doc":"Converts a charlist type into Nx' tuple format.","ref":"EXLA.Shape.html#charlist_to_dtype/1"},{"type":"function","title":"EXLA.Shape.dtype_to_charlist/1","doc":"Converts Nx's tuple format into charlist.","ref":"EXLA.Shape.html#dtype_to_charlist/1"},{"type":"function","title":"EXLA.Shape.make_shape/2","doc":"Creates a shape with the given type-size tuple and dimensions.","ref":"EXLA.Shape.html#make_shape/2"},{"type":"function","title":"EXLA.Shape.make_token_shape/0","doc":"Creates a token shape.","ref":"EXLA.Shape.html#make_token_shape/0"},{"type":"function","title":"EXLA.Shape.make_tuple_shape/1","doc":"Creates a tuple shape with the given shapes.","ref":"EXLA.Shape.html#make_tuple_shape/1"},{"type":"extras","title":"Rotating Image","doc":"# Rotating Image\n\n```elixir\nMix.install(\n  [\n    {:nx, github: \"elixir-nx/nx\", override: true, sparse: \"nx\"},\n    {:req, \"~> 0.3.5\"},\n    {:kino, \"~> 0.8.1\"},\n    {:exla, \"~> 0.5\"},\n    {:stb_image, \"~> 0.6\"}\n  ],\n  config: [\n    nx: [default_backend: EXLA.Backend]\n  ]\n)\n```","ref":"rotating-image.html"},{"type":"extras","title":"Preprocessing - Rotating Image","doc":"> Note: if you want to use TPUs/GPUs, you must set the XLA_TARGET environment variable.\n> See [`XLA_TARGET`](https://github.com/elixir-nx/xla#xla_target).\n\nPreprocessing a binary to the tensor representation and calculating shapes of the rotated image.\n\n```elixir\ndefmodule Preprocess do\n  def image_to_tensor(file_or_url) do\n    case URI.parse(file_or_url) do\n      %URI{scheme: nil} ->\n        img = StbImage.read_file!(file_or_url)\n        Nx.from_binary(img.data, {:u, 8}) |> Nx.reshape(img.shape)\n\n      _ ->\n        %{body: binary_image} = Req.get!(file_or_url)\n        img = StbImage.read_binary!(binary_image)\n        Nx.from_binary(img.data, {:u, 8}) |> Nx.reshape(img.shape)\n    end\n  end\n\n  defp calculate_new_side_length(side1, side2, angle, radians?) do\n    angle =\n      if radians?, do: angle, else: angle |> Nx.divide(180) |> Nx.multiply(Nx.Constants.pi())\n\n    sine = Nx.sin(angle)\n    cosine = Nx.cos(angle)\n\n    side1\n    |> Nx.multiply(cosine)\n    |> Nx.abs()\n    |> Nx.add(Nx.abs(Nx.multiply(side2, sine)))\n    |> Nx.add(1)\n    |> Nx.to_number()\n    |> round()\n  end\n\n  def get_new_height_and_width(height, width, angle, radians? \\\\ true) do\n    new_height = calculate_new_side_length(height, width, angle, radians?)\n    new_width = calculate_new_side_length(width, height, angle, radians?)\n    {new_height, new_width}\n  end\nend\n```","ref":"rotating-image.html#preprocessing"},{"type":"extras","title":"Rotating image - Rotating Image","doc":"```elixir\ndefmodule Rotate do\n  import Nx.Defn\n  @num_channels 4\n  # using three shears to avoid aliasing\n  # https://datagenetics.com/blog/august32013/index.html\n  defnp calculate_new_positions(\n          {y_image, x_image, _},\n          orig_ctr_height,\n          orig_ctr_width,\n          new_ctr_height,\n          new_ctr_width,\n          angle\n        ) do\n    k = Nx.iota({y_image * x_image})\n    i = Nx.as_type(k / x_image, {:u, 32})\n    j = Nx.remainder(k, x_image)\n\n    y = y_image - (orig_ctr_height + i + 1)\n\n    x = x_image - (orig_ctr_width + j + 1)\n\n    tangent = Nx.tan(angle / 2)\n\n    new_x = Nx.round(x - y * tangent)\n\n    new_y = Nx.round(Nx.sin(angle) * new_x + y)\n\n    new_x = Nx.round(new_x - new_y * tangent)\n\n    new_y = new_ctr_height - new_y\n    new_x = new_ctr_width - new_x\n\n    new_y = Nx.as_type(Nx.round(new_y), {:u, 32})\n    new_x = Nx.as_type(Nx.round(new_x), {:u, 32})\n\n    Nx.stack([new_y, new_x], axis: 1)\n  end\n\n  defnp calculate_ctr(coordinate) do\n    Nx.round((coordinate + 1) / 2 - 1)\n  end\n\n  defnp preprocess_position(pos, n) do\n    Nx.concatenate(\n      [\n        Nx.new_axis(pos, 1) |> Nx.tile([1, @num_channels, 1]),\n        Nx.iota({n, @num_channels, 1}, axis: 1)\n      ],\n      axis: 2\n    )\n    |> Nx.reshape({n * @num_channels, 3})\n  end\n\n  # new_image is a tensor with dims of rotated image filled with zeros\n\n  defn rotate_image_by_angle(image, angle, new_image, radians?) do\n    # if radians? is set to false than assuming that angle is given in degrees\n    angle = if radians?, do: angle, else: angle / 180 * Nx.Constants.pi()\n\n    {height, width, _} = image.shape\n    {new_height, new_width, _} = new_image.shape\n\n    orig_ctr_height = calculate_ctr(height)\n    orig_ctr_width = calculate_ctr(width)\n\n    new_ctr_height = calculate_ctr(new_height)\n    new_ctr_width = calculate_ctr(new_width)\n\n    pos =\n      calculate_new_positions(\n        image.shape,\n        orig_ctr_height,\n        orig_ctr_width,\n        new_ctr_height,\n        new_ctr_width,\n        angle\n      )\n\n    {n, 2} = Nx.shape(pos)\n\n    preprocessed_pos = preprocess_position(pos, n)\n    image = image |> Nx.reshape({n * @num_channels})\n\n    Nx.indexed_add(new_image, preprocessed_pos, image)\n  end\nend\n```","ref":"rotating-image.html#rotating-image"},{"type":"extras","title":"Example of usage - Rotating Image","doc":"```elixir\nnum_channels = 4\n\ndir = File.cwd!()\n\ndefault_input =\n  \"https://upload.wikimedia.org/wikipedia/commons/7/7f/Earth_fluent_design_reflective_surface_icon.png\"\n\ninput = Kino.Input.text(\"Image to rotate\", default: default_input)\nKino.render(input)\npath_to_file = Kino.Input.read(input)\n\n# set options\nradians? = 0\nangle = 90\n\n# rotate image by 70 degrees to the right\nimage = Preprocess.image_to_tensor(path_to_file)\n{height, width, _} = image.shape\n{new_height, new_width} = Preprocess.get_new_height_and_width(height, width, angle, radians?)\nnew_image = Nx.broadcast(Nx.tensor([0], type: {:u, 8}), {new_height, new_width, num_channels})\nrotate_image = EXLA.jit(&Rotate.rotate_image_by_angle/4)\nrotated_image = rotate_image.(image, angle, new_image, radians?)\n\n# convert the tensor back to stb_image and render it as png\nimg = StbImage.from_nx(rotated_image)\ncontent = StbImage.to_binary(img, :png)\n\n# display the rotated image\nKino.Image.new(content, :png)\n```","ref":"rotating-image.html#example-of-usage"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.7.3 (2024-07-01) - Changelog","doc":"* Update to latest Nx","ref":"changelog.html#v0-7-3-2024-07-01"},{"type":"extras","title":"v0.7.2 (2024-04-27) - Changelog","doc":"* Cast start indexes types when slicing a tensor","ref":"changelog.html#v0-7-2-2024-04-27"},{"type":"extras","title":"v0.7.1 (2024-02-27) - Changelog","doc":"* Add CustomCallOp for QR decomposition\n  * Minor improvements to the MLIR modules generated\n  * MLIR Context pooling for better concurrency","ref":"changelog.html#v0-7-1-2024-02-27"},{"type":"extras","title":"v0.7.0 (2024-02-22) - Changelog","doc":"* Update to latest Nx\n  * Introduce a `:mlir` based compiler and use it by default. The previous `:xla` based compiler is deprecatead. You can temporarily revert to the previous compiler by setting `config :exla, :compiler_mode, :xla`","ref":"changelog.html#v0-7-0-2024-02-22"},{"type":"extras","title":"v0.6.4 (2023-11-13) - Changelog","doc":"* Update to latest Nx\n  * Allow `:automatic_transfers` configuration on client\n  * Do not discard client/device in `EXLA.Backend` when it is host\n  * Always sort NaN last\n  * Improve the `:axes` option in `gather`, `indexed_add`, and `indexed_put`","ref":"changelog.html#v0-6-4-2023-11-13"},{"type":"extras","title":"v0.6.3 (2023-11-09) - Changelog","doc":"* Update to latest Nx\n  * Fix mixed device usage on EXLA.Backend","ref":"changelog.html#v0-6-3-2023-11-09"},{"type":"extras","title":"v0.6.1 (2023-09-12) - Changelog","doc":"* Update to latest Nx","ref":"changelog.html#v0-6-1-2023-09-12"},{"type":"extras","title":"v0.6.0 (2023-08-15) - Changelog","doc":"* Allow cross-device transfers on host\n  * Update dependencies to OpenXLA\n  * Update to latest Nx","ref":"changelog.html#v0-6-0-2023-08-15"},{"type":"extras","title":"v0.5.3 (2023-04-14) - Changelog","doc":"* Fix compilation issue on certain macOS caused by O3\n  * Fix optimization which would cause EXLA to return a complete tuple instead of a subset","ref":"changelog.html#v0-5-3-2023-04-14"},{"type":"extras","title":"v0.5.2 (2023-03-21) - Changelog","doc":"* Automatically transfer tensors between nodes","ref":"changelog.html#v0-5-2-2023-03-21"},{"type":"extras","title":"v0.5.1 (2023-02-18) - Changelog","doc":"* Support `top_k`","ref":"changelog.html#v0-5-1-2023-02-18"},{"type":"extras","title":"v0.5.0 (2023-02-10) - Changelog","doc":"* Optimize backend_transfer/backend_copy within EXLA backends\n  * Use relative symlinks on compilation whenever possible","ref":"changelog.html#v0-5-0-2023-02-10"},{"type":"extras","title":"v0.4.2 (2023-01-13) - Changelog","doc":"","ref":"changelog.html#v0-4-2-2023-01-13"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Automatically transfer from `:host` to other devices\n  * Support `lazy_transfers: :always` on `EXLA.jit/3` and `EXLA.compile/2`\n  * Run hooks concurrently once they have received the data","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Respect default `EXLA.Backend` client when jitting argumentless operations\n  * Do not pick client without devices when loading initial client\n  * Consider the first conditional of a `cond` part of the current scope","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.4.1 (2022-12-07) - Changelog","doc":"","ref":"changelog.html#v0-4-1-2022-12-07"},{"type":"extras","title":"Enhancements - Changelog","doc":"* [EXLA] Require Nx ~> 0.4.1\n  * [EXLA] Update `XLA` to TF2.11\n  * [EXLA.Defn] Send telemetry event after XLA compilation\n  * [EXLA.Op] Add optimization barriers as operations","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* [EXLA] Validate backend options\n  * [EXLA.Backend] Fix `Nx.{any,all}` with `:keep_axes`\n  * [EXLA.Backend] Make SVD return `V` instead of `transpose(V)`\n  * [EXLA.Backend] Preserve NaNs in `window` and `reduce` operations","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.4.0 (2022-10-25) - Changelog","doc":"","ref":"changelog.html#v0-4-0-2022-10-25"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support zero copy binaries\n  * Redirect group leader for EXLA hooks","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Always hoist `cond` expressions\n  * Fix conditional inside `Nx.map`","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.3.0 (2022-08-13) - Changelog","doc":"","ref":"changelog.html#v0-3-0-2022-08-13"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Support `debug: true` option on `defn` compiler\n  * Allow specifying preferred clients via the application environment\n  * Support new callbacks added in Nx v0.3.0","ref":"changelog.html#enhancements"},{"type":"extras","title":"Deprecations - Changelog","doc":"* Deprecate `set_as_nx_default`","ref":"changelog.html#deprecations"},{"type":"extras","title":"v0.2.3 (2022-07-05) - Changelog","doc":"","ref":"changelog.html#v0-2-3-2022-07-05"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fix predicate handling inside `cond`/`while`\n  * Set Nx backend globally","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.2.2 (2022-06-15) - Changelog","doc":"","ref":"changelog.html#v0-2-2-2022-06-15"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Fix invalid cache expiration when defn received functions as arguments","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.2.1 (2022-06-04) - Changelog","doc":"","ref":"changelog.html#v0-2-1-2022-06-04"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Implement `EXLA.Backend.to_batched_list/3`","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug fixes - Changelog","doc":"* Improve support for non-finite values in `EXLA` compiler\n  * Fix segmentation fault while deallocating tensors","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.2.0 (2022-04-28) - Changelog","doc":"First release.","ref":"changelog.html#v0-2-0-2022-04-28"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}